"""Main module."""
from email_verification.client import EmailVerificationClient
from email_verification.database import ResultDatabase


class EmailVerificationService:
    """Email verification service."""
    def __init__(self, api_key: str, db: ResultDatabase):
        """Initialize the service."""
        self.client = EmailVerificationClient(api_key)
        self.db = db

    def perform_verification(self, email: str) -> None:
        """Perform verification."""
        verification_result = self.client.verify_email(email)
        self.db.create_result(verification_result['email'], verification_result['verification_result'])
        print(f'Verification result for {email}: {verification_result["verification_result"]}')

    def show_all_results(self) -> None:
        """Show all results."""
        results = self.db.get_results()
        print('All results in the database:')
        for result in results:
            print(result)



"""Verify email using hunter."""
from typing import Dict, Union

import requests


class EmailVerificationClient:
    """Verify email using hunter."""
    def __init__(self, api_key: str):
        """Initialize the client."""
        self.api_key = api_key
        self.base_url = "https://api.hunter.io/v2/"

    def verify_email(self, email: str) -> Dict[str, Union[str, bool]]:
        """Verify email using hunter."""
        endpoint = f"email-verifier?email={email}&api_key={self.api_key}"
        response = requests.get(self.base_url + endpoint)
        response_data = response.json()

        if response.status_code == 200:
            return {'email': email, 'verification_result': response_data['data']['result']}
        return {'email': email, 'verification_result': f'Error: {response_data["errors"][0]["details"]}'}



# setup.py
from setuptools import setup, find_packages

setup(
    name='email_verification',
    version='0.1.0',
    packages=find_packages(),
    install_requires=[
        'requests>=2',
        'wemake-python-styleguide==0.18.0',

    ],
)



"""Database for email verification service."""
from typing import Dict, List, Union

EmailVerificationResult = Dict[str, Union[str, bool]]


class ResultDatabase:
    """Base class with CRUD operations."""

    def __init__(self):
        """Initialize the database."""
        self.checked_emails: List[EmailVerificationResult] = []

    def create_result(self, email: str, verification_result: Union[str, bool]) -> bool:
        """Create if not exist."""
        if self.exists(email):
            self.update_result(email, verification_result)
            return False
        self.checked_emails.append({'email': email, 'verification_result': verification_result})
        return True

    def get_results(self) -> List[EmailVerificationResult]:
        """Get all results."""
        return self.checked_emails

    def delete_result(self, email: str) -> None:
        """Delete if not exist."""
        self.checked_emails = [inner_email for inner_email in self.checked_emails if inner_email['email'] != email]

    def update_result(self, email: str, verification_result: Union[str, bool]) -> None:
        """Update if not exist."""
        for inner_email in self.checked_emails:
            if inner_email['email'] == email:
                inner_email['verification_result'] = verification_result

    def exists(self, email: str) -> bool:
        """Check if email is."""
        return any(inner_email['email'] == email for inner_email in self.checked_emails)
